@@ -4,7 +4,7 @@
 
   Author: Pekka Riikonen <priikone@silcnet.org>
 
-  Copyright (C) 1998 - 2003 Pekka Riikonen
+  Copyright (C) 1998 - 2005 Pekka Riikonen
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
   GNU General Public License for more details.
 
 */
-/* $Id: silcschedule.c,v 1.41.2.10 2003/10/30 20:22:52 priikone Exp $ */
+/* $Id: silcschedule.c,v 1.41.2.13 2005/05/06 11:20:26 priikone Exp $ */
 
 #include "silcincludes.h"
 #include "silcschedule_i.h"
@@ -607,6 +607,16 @@ bool silc_schedule_one(SilcSchedule schedule, int timeout_usecs)
     return FALSE;
   }
 
+  /* If the task queues aren't initialized or we aren't valid anymore
+     we will return */
+  if ((!schedule->fd_queue && !schedule->timeout_queue
+       && !schedule->generic_queue) || schedule->valid == FALSE) {
+    SILC_LOG_DEBUG(("Scheduler not valid anymore, exiting"));
+    if (!schedule->is_locked)
+      SILC_SCHEDULE_UNLOCK(schedule);
+    return FALSE;
+  }
+
   /* Calculate next timeout for silc_select(). This is the timeout value
      when at earliest some of the timeout tasks expire. */
   silc_mutex_lock(schedule->timeout_queue->lock);
@@ -903,6 +913,7 @@ void silc_schedule_set_listen_fd(SilcSchedule schedule, SilcUInt32 fd,
     if (schedule->fd_list[i].fd == fd) {
       schedule->fd_list[i].fd = fd;
       schedule->fd_list[i].events = mask;
+      schedule->fd_list[i].revents = 0;
       if (i > schedule->last_fd)
 	schedule->last_fd = i;
       found = TRUE;
@@ -918,6 +929,7 @@ void silc_schedule_set_listen_fd(SilcSchedule schedule, SilcUInt32 fd,
       if (schedule->fd_list[i].events == 0) {
 	schedule->fd_list[i].fd = fd;
 	schedule->fd_list[i].events = mask;
+	schedule->fd_list[i].revents = 0;
 	if (i > schedule->last_fd)
 	  schedule->last_fd = i;
 	if (send_events) {
@@ -944,8 +956,9 @@ void silc_schedule_unset_listen_fd(SilcSchedule schedule, SilcUInt32 fd)
     if (schedule->fd_list[i].fd == fd) {
       schedule->fd_list[i].fd = 0;
       schedule->fd_list[i].events = 0;
-      if (schedule->last_fd == i)
-	schedule->last_fd = schedule->max_fd - 1;
+      schedule->fd_list[i].revents = 0;
+      if (schedule->last_fd == i && i > 0)
+	schedule->last_fd = i - 1;
       break;
     }
 
